package de.mdelab.expressions.interpreter.ocl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.ocl.OCLInput;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.ecore.OCL;

public class OCLWithLibraries extends OCL {

	private static final String LINE_SEPARATOR = "\n";
	
	private final Collection<String> libraryIds;

	protected OCLWithLibraries(	final EcoreEnvironmentFactory envFactory,
								final Map<String, String> libraries ) 
	throws ParserException {
		super( envFactory );
		
		libraryIds = initLibraries( libraries );
	}

	protected OCLWithLibraries(	final EcoreEnvironmentFactory envFactory,
								final Map<String, String> libraries,
								final Resource resource ) 
	throws ParserException {
		super( envFactory, resource );
		
		libraryIds = initLibraries( libraries );
	}
	
	private Collection<String> initLibraries( final Map<String, String> libraries )
	throws ParserException {
		final Collection<String> libIds = new ArrayList<String>();
		libIds.addAll( libraries.keySet() );
		parseLibraries( libraries.values() );
		
		return libIds;
	}
	
	protected Collection<String> getLibraryIds() {
		return libraryIds;
	}
	
	public Object getId() {
		return getLibraryIds();
	}
	
	private void parseLibraries( final Collection<String> libraryTexts ) 
	throws ParserException {
        for ( final String libraryText : libraryTexts ) {
        	final String parsedText;
        	
        	if ( libraryText.startsWith( "MainModel : " ) ) {
        		final int endLineIndex = libraryText.indexOf( LINE_SEPARATOR );
        		
        		if ( endLineIndex > 0 ) {
        			parsedText = libraryText.substring( endLineIndex + LINE_SEPARATOR.length() + 1 );
        		}
        		else {
        			parsedText = libraryText;
        		}
        	}
        	else {
    			parsedText = libraryText;
        	}
        	
        	parse( new OCLInput( parsedText.toString() ) );
        }
	}
}
